Package: phytools
Version: 2.5-2
Date: 2025-09-18
Title: Phylogenetic Tools for Comparative Biology (and Other Things)
Authors@R: person(given = c("Liam", "J."),
	family = "Revell",
	role = c("aut", "cre"),
	email = "liam.revell@umb.edu")
Depends: R (>= 3.5.0), ape (>= 5.7), maps
Imports: clusterGeneration, coda, combinat, DEoptim, doParallel, expm,
        foreach, graphics, grDevices, MASS, methods, mnormt, nlme,
        numDeriv, optimParallel, parallel, phangorn (>= 2.3.1),
        scatterplot3d, stats, utils
Suggests: animation, geiger, plotrix, RColorBrewer, rgl
ZipData: no
Description: A wide range of methods for phylogenetic analysis - concentrated in phylogenetic comparative biology, but also including numerous techniques for visualizing, analyzing, manipulating, reading or writing, and even inferring phylogenetic trees. Included among the functions in phylogenetic comparative biology are various for ancestral state reconstruction, model-fitting, and simulation of phylogenies and trait data. A broad range of plotting methods for phylogenies and comparative data include (but are not restricted to) methods for mapping trait evolution on trees, for projecting trees into phenotype space or a onto a geographic map, and for visualizing correlated speciation between trees. Lastly, numerous functions are designed for reading, writing, analyzing, inferring, simulating, and manipulating phylogenetic trees and comparative data. For instance, there are functions for computing consensus phylogenies from a set, for simulating phylogenetic trees and data under a range of models, for randomly or non-randomly attaching species or clades to a tree, as well as for a wide range of other manipulations and analyses that phylogenetic biologists might find useful in their research.
License: GPL (>= 2)
URL: https://github.com/liamrevell/phytools
Packaged: 2025-09-18 20:30:20 UTC; liamj
Repository: CRAN
Date/Publication: 2025-09-19 05:10:02 UTC
NeedsCompilation: no
Author: Liam J. Revell [aut, cre]
Maintainer: Liam J. Revell <liam.revell@umb.edu>
