## names exported by phytools
export(add.arrow, add.color.bar, add.everywhere, add.random, add.simmap.legend)
export(add.species.to.genus, aic.w, allFurcTrees, allRotations, anc.Bayes, anc.ML)
export(anc.trend, ancr, ancThresh, ansi_phylo, applyBranchLengths, arc.cladelabels, as.multiPhylo)
export(as.prcomp, as.princomp, as.Qmatrix, ave.rates, averageTree)
export(backbone.toPhylo, bd, bmPlot, bind.tip, bind.tree.simmap, biplot.phyl.pca)
export(bounded_bm, branching.diffusion, brownie.lite, brownieREML)
export(cladebox, cladelabels, coef.phyl.RMA, collapse.to.star, collapseTree, compare.chronograms)
export(consensus.edges, contMap, cophylo, countSimmap, compute.mr, cotangleplot, cospeciation)
export(ctt)
export(density.anc.Bayes, density.multiSimmap, densityMap, densityTree, describe.simmap)
export(di2multi.simmap, dot.legend, dotTree, drop.clade, drop.leaves, drop.tip.contMap)
export(drop.tip.densityMap, drop.tip.multiSimmap, drop.tip.simmap, drop.tip.singleton, Dtest)
export(edge.widthMap, edgelabels.cophylo, edgeProbs, errorbar.contMap, estDiversity)
export(evol.rate.mcmc, evol.vcv, evolvcv.lite, exhaustiveMP, expand.clade, export.as.xml)
export(extract.clade.simmap, extract.strahlerNumber)
export(fancyTree, fastAnc, fastBM, fastDist, fastHeight, fastMRCA, findMRCA, fit.bd)
export(fit.yule, fitBayes, fitcontMk, fitfnMk, fitgammaMk, fitHRM, fitMk, fitMk.parallel, fitmultiBM)
export(fitmultiMk, fitmultiTrend, fitpolyMk, fitDiversityModel, fitPagel, fitsemiThresh, fitThresh)
export(force.ultrametric)
export(gamma_pruning, gammatest, genus.to.species.tree, genSeq, geo.palette, geo.legend, get.treepos)
export(getCladesofSize, getDescendants, getExtant, getExtinct, getnode, getParent)
export(getSisters, getStates, graph.polyMk, gtt)
export(hide.hidden)
export(keep.tip.contMap, keep.tip.densityMap, keep.tip.multiSimmap, keep.tip.simmap)
export(labelnodes, ladderize.simmap, lambda.transform, lik.bd, likMlambda)
export(likSurface.rateshift, linklabels, locate.fossil, locate.yeti)
export(ls.consensus, ls.tree, ltt, ltt95, ltt.multiPhylo, ltt.multiSimmap, ltt.phylo, ltt.simmap)
export(make.era.map, make.simmap, make.transparent, map.overlap, Map.Overlap)
export(map.to.singleton, mapped.states, markChanges, matchLabels, matchNodes, mccr)
export(mcmcMk, mergeMappedStates, midpoint.root, midpoint_root, minRotate, minSplit, minTreeDist)
export(modified.Grafen, mrp.supertree, multi.mantel, multiC, multiOU, multiRF)
export(multirateBM)
export(nodeheight, nodeHeights, nodelabels.cophylo, node.paths)
export(optim.phylo.ls, orderMappedEdge)
export(paintBranches, paintSubTree, paste.tree, pbtree, pgls.Ives, pgls.SEy, phenogram)
export(phenogram95, phyl.cca, phyl.pairedttest, phyl.pca, phyl.resid, phyl.RMA)
export(phyl.vcv, phylo.heatmap, phylo.impute, phylo.toBackbone, phylo.to.map, phylANOVA)
export(phyloDesign, phylomorphospace, phylomorphospace3d, phyloScattergram, phylosig)
export(plot.anc.Bayes, plot.ancr, plot.changesMap, plot.contMap, plot.cophylo, plot.densityMap)
export(plot.edge.widthMap, plot.fitMk, plot.fitPagel, plot.gfit, plot.phyl.RMA)
export(plot.phylo.to.map, plot.Qmatrix, plot.simBMphylo, plotBranchbyTrait, plot.phylosig)
export(plotFanTree.wTraits, plotSimmap, plotThresh, plotTree, plotTree.barplot, plotTree.boxplot)
export(plotTree.datamatrix, plotTree.errorbars, plotTree.lollipop, plotTree.singletons, plotTree.splits)
export(plotTree.wBars, posterior.evolrate, posthoc, posthoc.ratebytree, print.Qmatrix)
export(project.phylomorphospace, pscore)
export(ratebystate, ratebytree, rateshift, read.newick, read.simmap, readNexus)
export(reorder.backbonePhylo, reorderSimmap, rep.multiPhylo, rep.phylo, repPhylo, reroot)
export(rerootingMethod, rescale, rescale.multiSimmap, rescale.simmap, rescaleSimmap, resolveAllNodes)
export(resolveNode, rootedge.to.singleton, rotate.multi, rotateNodes, roundBranches, roundPhylogram)
export(rstate)
export(sampleFrom, scores, scores.phyl.pca, setMap, sigmoidPhylogram, sim.absorbing, sim.corrs, sim.ctt)
export(sim.history, sim.Mk, sim.multiCtt, sim.multiMk, sim.multiTrend, sim.ratebystate, sim.rates)
export(sim.reflective, simBMphylo, simmap, skewers, splinePhylogram, splitEdgeColor, splitplotTree, splitTree)
export(starTree, strahlerNumber)
export(threshBayes, threshDIC, threshState, tiplabels.cophylo, tipRotate, to.matrix)
export(tree.grow, treeSlice)
export(untangle)
export(vcvPhylo)
export(write.simmap, writeAncestors, writeNexus)

## S3 methods
S3method(plot, densityMap)
S3method(plot, contMap)
S3method(plot, phylo.to.map)
S3method(plot, backbonePhylo)
S3method(reorder, backbonePhylo)
S3method(print, backbonePhylo)
S3method(print, phyl.pca)
S3method(summary, phyl.pca)
S3method(biplot, phyl.pca)
S3method(print, brownie.lite)
S3method(print, densityMap)
S3method(print, contMap)
S3method(print, rateshift)
S3method(logLik, rateshift)
S3method(print, evol.vcv)
S3method(plot, ltt95)
S3method(print, ltt95)
S3method(plot, describe.simmap)
S3method(print, describe.simmap)
S3method(rep, phylo)
S3method(rep, multiPhylo)
S3method(print, fitPagel)
S3method(print, ltt)
S3method(print, multiLtt)
S3method(plot, ltt)
S3method(plot, multiLtt)
S3method(plot, cophylo)
S3method(print, cophylo)
S3method(plot, simmap)
S3method(print, simmap)
S3method(summary, simmap)
S3method(plot, multiSimmap)
S3method(print, multiSimmap)
S3method(summary, multiSimmap)
S3method(reorder, simmap)
S3method(plot, rateshift)
S3method(print, fastAnc)
S3method(print, anc.ML)
S3method(print, fitMk)
S3method(summary, fitMk)
S3method(logLik, fitMk)
S3method(print, brownieREML)
S3method(print, phylo.to.map)
S3method(as.phylo, simmap)
S3method(as.multiPhylo, multiSimmap)
S3method(as.multiPhylo, phylo)
S3method(plot, fitPagel)
S3method(plot, fitMk)
S3method(plot, gfit)
S3method(plot, phyl.pca)
S3method(summary, cophylo)
S3method(print, multiCophylo)
S3method(plot, multiCophylo)
S3method(logLik, simmap)
S3method(logLik, multiSimmap)
S3method(plot, cospeciation)
S3method(print, cospeciation)
S3method(density, multiSimmap)
S3method(plot, changesMap)
S3method(print, changesMap)
S3method(plot, phyloScattergram)
S3method(print, phyloScattergram)
S3method(di2multi, simmap)
S3method(print, aic.w)
S3method(print, phyl.RMA)
S3method(coef, phyl.RMA)
S3method(residuals, phyl.RMA)
S3method(plot, phyl.RMA)
S3method(print, ratebytree)
S3method(print, evolvcv.lite)
S3method(print, expand.clade)
S3method(plot, expand.clade)
S3method(posthoc, ratebytree)
S3method(print, posthoc.ratebytree)
S3method(AIC, ratebytree)
S3method(print, fit.bd)
S3method(logLik, fit.bd)
S3method(print, anc.trend)
S3method(logLik, anc.trend)
S3method(print, ancThresh)
S3method(plot, ancThresh)
S3method(print, evol.rate.mcmc)
S3method(print, threshBayes)
S3method(plot, threshBayes)
S3method(print, rerootingMethod)
S3method(plot, rerootingMethod)
S3method(logLik, rerootingMethod)
S3method(print, fitBayes)
S3method(plot, fitBayes)
S3method(print, ratebystate)
S3method(print, phylANOVA)
S3method(print, pgls.Ives)
S3method(logLik, pgls.Ives)
S3method(print, phyl.cca)
S3method(print, anc.Bayes)
S3method(plot, anc.Bayes)
S3method(print, phyl.pairedttest)
S3method(print, multi.mantel)
S3method(residuals, multi.mantel)
S3method(fitted, multi.mantel)
S3method(print, ctt)
S3method(print, multiCtt)
S3method(plot, ctt)
S3method(plot, multiCtt)
S3method(print, fitmultiMk)
S3method(summary, fitmultiMk)
S3method(logLik, fitmultiMk)
S3method(plot, gtt)
S3method(print, gtt)
S3method(print, mcmcMk)
S3method(print, Dtest)
S3method(print, mccr)
S3method(plot, mccr)
S3method(print, fitpolyMk)
S3method(logLik, fitpolyMk)
S3method(plot, fitpolyMk)
S3method(plot, mcmcMk)
S3method(summary, mcmcMk)
S3method(density, mcmcMk)
S3method(print, density.mcmcMk)
S3method(plot, density.mcmcMk)
S3method(multi2di, simmap)
S3method(di2multi, multiSimmap)
S3method(multi2di, multiSimmap)
S3method(di2multi, contMap)
S3method(multi2di, contMap)
S3method(di2multi, densityMap)
S3method(multi2di, densityMap)
S3method(summary, evol.rate.mcmc)
S3method(print, summary.evol.rate.mcmc)
S3method(plot, summary.evol.rate.mcmc)
S3method(setMap, contMap)
S3method(setMap, densityMap)
S3method(setMap, phyloScattergram)
S3method(print, fitDiversityModel)
S3method(logLik, fitDiversityModel)
S3method(print, gammatest)
S3method(print, geo.legend)
S3method(print, geo.palette)
S3method(scores, phyl.pca)
S3method(logLik, brownie.lite)
S3method(plot, simBMphylo)
S3method(print, simBMphylo)
S3method(print, phylosig)
S3method(plot, phylosig)
S3method(density, threshBayes)
S3method(plot, density.threshBayes)
S3method(as.Qmatrix, fitMk)
S3method(print, Qmatrix)
S3method(density, anc.Bayes)
S3method(summary, anc.Bayes)
S3method(plot, Qmatrix)
S3method(logLik, anc.ML)
S3method(print, edge.widthMap)
S3method(plot, edge.widthMap)
S3method(plot, fitHRM)
S3method(print, fitHRM)
S3method(as.Qmatrix, corhmm)
S3method(plot, multirateBM)
S3method(print, multirateBM)
S3method(logLik, multirateBM)
S3method(plot, multirateBM_plot)
S3method(setMap, multirateBM_plot)
S3method(print, multirateBM_plot)
S3method(as.princomp, phyl.pca)
S3method(ltt, phylo)
S3method(ltt, multiPhylo)
S3method(ltt, simmap)
S3method(plot, ltt.simmap)
S3method(print, ltt.simmap)
S3method(ltt, multiSimmap)
S3method(print, ltt.multiSimmap)
S3method(plot, ltt.multiSimmap)
S3method(as.Qmatrix, fitpolyMk)
S3method(anova, evolvcv.lite)
S3method(anova, fitPagel)
S3method(anova, fit.bd)
S3method(anova, fitMk)
S3method(as.Qmatrix, matrix)
S3method(drop.tip, singleton)
S3method(drop.tip, simmap)
S3method(drop.tip, contMap)
S3method(drop.tip, densityMap)
S3method(drop.tip, multiSimmap)
S3method(keep.tip, contMap)
S3method(keep.tip, densityMap)
S3method(keep.tip, multiSimmap)
S3method(keep.tip, simmap)
S3method(anova, fitpolyMk)
S3method(rescale, simmap)
S3method(rescale, multiSimmap)
S3method(simmap, fitMk)
S3method(simmap, anova.fitMk)
S3method(simmap, fitpolyMk)
S3method(Ntip, contMap)
S3method(Nnode, contMap)
S3method(Nedge, contMap)
S3method(Ntip, densityMap)
S3method(Nnode, densityMap)
S3method(Nedge, densityMap)
S3method(simmap, simmap)
S3method(ancr, fitMk)
S3method(logLik, ancr)
S3method(print, ancr)
S3method(ancr, fitpolyMk)
S3method(ancr, fitHRM)
S3method(ancr, anova.fitMk)
S3method(anova, fitHRM)
S3method(hide.hidden, ancr)
S3method(plot, ancr)
S3method(c, simmap)
S3method(c, multiSimmap)
S3method(midpoint, root)
S3method(as.Qmatrix, ace)
S3method(as.prcomp, phyl.pca)
S3method(print, fitgammaMk)
S3method(logLik,fitgammaMk)
S3method(anova, fitgammaMk)
S3method(ancr, fitgammaMk)
S3method(as.Qmatrix, fitgammaMk)
S3method(plot, fitgammaMk)
S3method(ancr, bounded_bm)
S3method(print, bounded_bm)
S3method(print, ancr.bounded_bm)
S3method(logLik, bounded_bm)
S3method(print, fitfnMk)
S3method(plot, fitfnMk)
S3method(logLik, fitfnMk)
S3method(ancr, fitThresh)
S3method(logLik, fitThresh)
S3method(print, fitThresh)
S3method(print, fitmultiBM)
S3method(logLik, fitmultiBM)
S3method(ancr, fitmultiBM)
S3method(print, ancr.fitmultiBM)
S3method(anova, rateshift)
S3method(Ntip, cophylo)
S3method(Nnode, cophylo)
S3method(Nedge, cophylo)
S3method(print, fitsemiThresh)
S3method(logLik, fitsemiThresh)
S3method(print, fitmultiTrend)
S3method(print, fitcontMk)
S3method(plot, fitcontMk)

## default methods
S3method(ancr, default)
S3method(as.Qmatrix, default)
S3method(as.princomp, default)
S3method(hide.hidden, default)
S3method(ltt, default)
S3method(rescale, default)
S3method(scores, default)
S3method(setMap, default)
S3method(simmap, default)
S3method(as.prcomp, default)

## names imported from other packages
importFrom(ape, .PlotPhyloEnv, .uncompressTipLabel, ace, all.equal.phylo, as.DNAbin) 
importFrom(ape, as.phylo, bind.tree, branching.times, collapse.singles, consensus)
importFrom(ape, compute.brlen, cophenetic.phylo, corBrownian, di2multi, dist.dna) 
importFrom(ape, dist.nodes, drop.tip, edgelabels, extract.clade, getMRCA, is.binary)
importFrom(ape, is.monophyletic, is.rooted, is.ultrametric, keep.tip, ladderize, matexpo, mrca)
importFrom(ape, multi2di, nodelabels, Nedge, Nnode, Ntip, pic, plot.phylo, prop.part)
importFrom(ape, read.nexus, read.tree, reorder.phylo, root.phylo, rotate, rtree, stree)
importFrom(ape, tiplabels, unroot, vcv, vcv.phylo, write.tree)
importFrom(coda, as.mcmc, HPDinterval)
importFrom(combinat, permn)
importFrom(clusterGeneration, genPositiveDefMat)
importFrom(DEoptim, DEoptim)
importFrom(doParallel, registerDoParallel)
importFrom(expm, expm)
importFrom(foreach, "%dopar%", foreach)
importFrom(graphics, strwidth, par, segments, locator, lines, text, strheight, symbols) 
importFrom(graphics, plot, layout, plot.new, title, axis, points, plot.window, polygon) 
importFrom(graphics, rect, curve, image, mtext, arrows, barplot, boxplot, contour, hist)
importFrom(graphics, abline, legend, grid, clip, box)
importFrom(grDevices, palette, palette.colors, colors, dev.hold, dev.flush, rainbow, heat.colors, gray)
importFrom(grDevices, colorRamp, colorRampPalette, dev.new, rgb, pdf, dev.off, col2rgb, dev.size)
importFrom(grDevices, xy.coords, dev.cur, hcl.colors)
importFrom(maps, map)
importFrom(MASS, ginv)
importFrom(methods, hasArg, is)
importFrom(mnormt, dmnorm, pd.solve)
importFrom(nlme, gls, varFixed)
importFrom(numDeriv, hessian)
importFrom(optimParallel, optimParallel)
importFrom(parallel, detectCores, makeCluster, stopCluster)
importFrom(phangorn, allTrees, Ancestors, as.phyDat, dist.hamming, midpoint, NJ, nni)
importFrom(phangorn, nnls.tree, optim.parsimony, parsimony, phyDat, pratchet, treedist) 
importFrom(phangorn, RF.dist, KF.dist, path.dist, SPR.dist, Children, Descendants) 
importFrom(phangorn, threshStateC)
importFrom(scatterplot3d, scatterplot3d)
importFrom(stats, cophenetic, reorder, runif, setNames, optim, dexp, dnorm, rnorm, var)
importFrom(stats, nlminb, biplot, pchisq, rexp, optimize, logLik, as.dist, pnorm, median) 
importFrom(stats, cor, aggregate, rgamma, dgamma, lm, rbinom, rgeom, pt, anova, p.adjust) 
importFrom(stats, screeplot, rchisq, optimHess, rmultinom, sd, dunif, dist, plogis) 
importFrom(stats, density, coef, cmdscale, AIC, smooth.spline, predict, qgamma, qnorm)
importFrom(utils, flush.console, head, installed.packages, str, capture.output, combn)

## restore if plotrix unorphaned on CRAN
## importFrom(plotrix, arctext, draw.arc, draw.circle, draw.ellipse, textbox)
